@@ -52,8 +52,9 @@ SoapyRTLSDR::SoapyRTLSDR(const SoapySDR::Kwargs &args)
     bufferLength = DEFAULT_BUFFER_LENGTH;
 
     iqSwap = false;
-    agcMode = false;
+    gainMode = false;
     offsetMode = false;
+    digitalAGC = false;
 
     bufferedElems = 0;
     resetBuffer = false;
@@ -141,7 +142,25 @@ std::string SoapyRTLSDR::getDriverKey(void) const
 
 std::string SoapyRTLSDR::getHardwareKey(void) const
 {
-    return "RTLSDR";
+    switch (rtlsdr_get_tuner_type(dev))
+    {
+    case RTLSDR_TUNER_UNKNOWN:
+        return "UNKNOWN";
+    case RTLSDR_TUNER_E4000:
+        return "E4000";
+    case RTLSDR_TUNER_FC0012:
+        return "FC0012";
+    case RTLSDR_TUNER_FC0013:
+        return "FC0013";
+    case RTLSDR_TUNER_FC2580:
+        return "FC2580";
+    case RTLSDR_TUNER_R820T:
+        return "R820T";
+    case RTLSDR_TUNER_R828D:
+        return "R828D";
+    default:
+        return "OTHER";
+    }
 }
 
 SoapySDR::Kwargs SoapyRTLSDR::getHardwareInfo(void) const
@@ -245,14 +264,14 @@ bool SoapyRTLSDR::hasGainMode(const int direction, const size_t channel) const
 
 void SoapyRTLSDR::setGainMode(const int direction, const size_t channel, const bool automatic)
 {
-    agcMode = automatic;
-    SoapySDR_logf(SOAPY_SDR_DEBUG, "Setting RTL-SDR AGC: %s", automatic ? "Automatic" : "Manual");
-    rtlsdr_set_agc_mode(dev, agcMode ? 1 : 0);
+    gainMode = automatic;
+    SoapySDR_logf(SOAPY_SDR_DEBUG, "Setting RTL-SDR gain mode: %s", automatic ? "Automatic" : "Manual");
+    rtlsdr_set_tuner_gain_mode(dev, gainMode ? 0 : 1);
 }
 
 bool SoapyRTLSDR::getGainMode(const int direction, const size_t channel) const
 {
-    return SoapySDR::Device::getGainMode(direction, channel);
+    return gainMode;
 }
 
 void SoapyRTLSDR::setGain(const int direction, const size_t channel, const double value)
@@ -522,6 +541,16 @@ SoapySDR::ArgInfoList SoapyRTLSDR::getSettingInfo(void) const
 
     setArgs.push_back(iqSwapArg);
 
+    SoapySDR::ArgInfo digitalAGCArg;
+
+    digitalAGCArg.key = "digital_agc";
+    digitalAGCArg.value = "false";
+    digitalAGCArg.name = "Digital AGC";
+    digitalAGCArg.description = "RTL-SDR digital AGC Mode";
+    digitalAGCArg.type = SoapySDR::ArgInfo::BOOL;
+
+    setArgs.push_back(digitalAGCArg);
+
     SoapySDR_logf(SOAPY_SDR_DEBUG, "SETARGS?");
 
     return setArgs;
@@ -553,6 +582,12 @@ void SoapyRTLSDR::writeSetting(const std::string &key, const std::string &value)
         SoapySDR_logf(SOAPY_SDR_DEBUG, "RTL-SDR offset_tune mode: %s", offsetMode ? "true" : "false");
         rtlsdr_set_offset_tuning(dev, offsetMode ? 1 : 0);
     }
+    else if (key == "digital_agc")
+    {
+        digitalAGC = (value == "true") ? true : false;
+        SoapySDR_logf(SOAPY_SDR_DEBUG, "RTL-SDR digital agc mode: %s", digitalAGC ? "true" : "false");
+        rtlsdr_set_agc_mode(dev, digitalAGC ? 1 : 0);
+    }
 }
 
 std::string SoapyRTLSDR::readSetting(const std::string &key) const
@@ -563,6 +598,8 @@ std::string SoapyRTLSDR::readSetting(const std::string &key) const
         return iqSwap?"true":"false";
     } else if (key == "offset_tune") {
         return offsetMode?"true":"false";
+    } else if (key == "digital_agc") {
+        return digitalAGC?"true":"false";
     }
 
     SoapySDR_logf(SOAPY_SDR_WARNING, "Unknown setting '%s'", key.c_str());
